@@ -36,7 +36,7 @@ class AuthHandler:
         :class:`~acme.challenges.Challenge` types
     :type auth: certbot.interfaces.Authenticator
 
-    :ivar acme.client.BackwardsCompatibleClientV2 acme_client: ACME client API.
+    :ivar acme.client.ClientV2 acme_client: ACME client API.
 
     :ivar account: Client's Account
     :type account: :class:`certbot._internal.account.Account`
@@ -55,7 +55,8 @@ class AuthHandler:
 
     def handle_authorizations(self, orderr: messages.OrderResource,
                               config: configuration.NamespaceConfig, best_effort: bool = False,
-                              max_retries: int = 30) -> List[messages.AuthorizationResource]:
+                              max_retries: int = 30,
+                              max_time_mins: float = 30) -> List[messages.AuthorizationResource]:
         """
         Retrieve all authorizations, perform all challenges required to validate
         these authorizations, then poll and wait for the authorization to be checked.
@@ -63,6 +64,7 @@ class AuthHandler:
         :param certbot.configuration.NamespaceConfig config: current Certbot configuration
         :param bool best_effort: if True, not all authorizations need to be validated (eg. renew)
         :param int max_retries: maximum number of retries to poll authorizations
+        :param float max_time_mins: maximum time (in minutes) to poll authorizations
         :returns: list of all validated authorizations
         :rtype: List
 
@@ -103,7 +105,7 @@ class AuthHandler:
 
             # Wait for authorizations to be checked.
             logger.info('Waiting for verification...')
-            self._poll_authorizations(authzrs, max_retries, best_effort)
+            self._poll_authorizations(authzrs, max_retries, max_time_mins, best_effort)
 
             # Keep validated authorizations only. If there is none, no certificate can be issued.
             authzrs_validated = [authzr for authzr in authzrs
@@ -143,11 +145,11 @@ class AuthHandler:
         return (deactivated, failed)
 
     def _poll_authorizations(self, authzrs: List[messages.AuthorizationResource], max_retries: int,
-                             best_effort: bool) -> None:
+                             deadline_minutes: float, best_effort: bool) -> None:
         """
         Poll the ACME CA server, to wait for confirmation that authorizations have their challenges
         all verified. The poll may occur several times, until all authorizations are checked
-        (valid or invalid), or after a maximum of retries.
+        (valid or invalid), or a maximum of retries, or the polling deadline is reached.
         """
         if not self.acme:
             raise errors.Error("No ACME client defined, cannot poll authorizations.")
@@ -156,6 +158,7 @@ class AuthHandler:
                                           Optional[Response]]] = {index: (authzr, None)
                             for index, authzr in enumerate(authzrs)}
         authzrs_failed_to_report = []
+        deadline = datetime.datetime.now() + datetime.timedelta(minutes=deadline_minutes)
         # Give an initial second to the ACME CA server to check the authorizations
         sleep_seconds: float = 1
         for _ in range(max_retries):
@@ -184,7 +187,7 @@ class AuthHandler:
             authzrs_to_check = {index: (authzr, resp) for index, (authzr, resp)
                                 in authzrs_to_check.items()
                                 if authzr.body.status == messages.STATUS_PENDING}
-            if not authzrs_to_check:
+            if not authzrs_to_check or datetime.datetime.now() > deadline:
                 # Polling process is finished, we can leave the loop
                 break
 
@@ -196,6 +199,9 @@ class AuthHandler:
             retry_after = max(self.acme.retry_after(resp, 3)
                               for _, resp in authzrs_to_check.values()
                               if resp is not None)
+            # Whatever Retry-After the ACME server requests, the polling must not take
+            # longer than the overall deadline (https://github.com/certbot/certbot/issues/9526).
+            retry_after = min(retry_after, deadline)
             sleep_seconds = (retry_after - datetime.datetime.now()).total_seconds()
 
         # In case of failed authzrs, create a report to the user.
@@ -226,15 +232,10 @@ class AuthHandler:
             logger.info("Performing the following challenges:")
         for authzr in pending_authzrs:
             authzr_challenges = authzr.body.challenges
-            if self.acme.acme_version == 1:
-                combinations = authzr.body.combinations
-            else:
-                combinations = tuple((i,) for i in range(len(authzr_challenges)))
 
             path = gen_challenge_path(
                 authzr_challenges,
-                self._get_chall_pref(authzr.body.identifier.value),
-                combinations)
+                self._get_chall_pref(authzr.body.identifier.value))
 
             achalls.extend(self._challenge_factory(authzr, path))
 
@@ -387,12 +388,9 @@ def challb_to_achall(challb: messages.ChallengeBody, account_key: josepy.JWK,
 
 
 def gen_challenge_path(challbs: List[messages.ChallengeBody],
-                       preferences: List[Type[challenges.Challenge]],
-                       combinations: Tuple[Tuple[int, ...], ...]) -> Tuple[int, ...]:
+                       preferences: List[Type[challenges.Challenge]]) -> Tuple[int, ...]:
     """Generate a plan to get authority over the identity.
 
-    .. todo:: This can be possibly be rewritten to use resolved_combinations.
-
     :param tuple challbs: A tuple of challenges
         (:class:`acme.messages.Challenge`) from
         :class:`acme.messages.AuthorizationResource` to be
@@ -402,10 +400,6 @@ def gen_challenge_path(challbs: List[messages.ChallengeBody],
     :param list preferences: List of challenge preferences for domain
         (:class:`acme.challenges.Challenge` subclasses)
 
-    :param tuple combinations: A collection of sets of challenges from
-        :class:`acme.messages.Challenge`, each of which would
-        be sufficient to prove possession of the identifier.
-
     :returns: list of indices from ``challenges``.
     :rtype: list
 
@@ -413,21 +407,6 @@ def gen_challenge_path(challbs: List[messages.ChallengeBody],
         path cannot be created that satisfies the CA given the preferences and
         combinations.
 
-    """
-    if combinations:
-        return _find_smart_path(challbs, preferences, combinations)
-    return _find_dumb_path(challbs, preferences)
-
-
-def _find_smart_path(challbs: List[messages.ChallengeBody],
-                     preferences: List[Type[challenges.Challenge]],
-                     combinations: Tuple[Tuple[int, ...], ...]
-                     ) -> Tuple[int, ...]:
-    """Find challenge path with server hints.
-
-    Can be called if combinations is included. Function uses a simple
-    ranking system to choose the combo with the lowest cost.
-
     """
     chall_cost = {}
     max_cost = 1
@@ -441,6 +420,8 @@ def _find_smart_path(challbs: List[messages.ChallengeBody],
     # Set above completing all of the available challenges
     best_combo_cost = max_cost
 
+    combinations = tuple((i,) for i in range(len(challbs)))
+
     combo_total = 0
     for combo in combinations:
         for challenge_index in combo:
@@ -459,28 +440,6 @@ def _find_smart_path(challbs: List[messages.ChallengeBody],
     return best_combo
 
 
-def _find_dumb_path(challbs: List[messages.ChallengeBody],
-                    preferences: List[Type[challenges.Challenge]]) -> Tuple[int, ...]:
-    """Find challenge path without server hints.
-
-    Should be called if the combinations hint is not included by the
-    server. This function either returns a path containing all
-    challenges provided by the CA or raises an exception.
-
-    """
-    path = []
-    for i, challb in enumerate(challbs):
-        # supported is set to True if the challenge type is supported
-        supported = next((True for pref_c in preferences
-                          if isinstance(challb.chall, pref_c)), False)
-        if supported:
-            path.append(i)
-        else:
-            raise _report_no_chall_path(challbs)
-
-    return tuple(path)
-
-
 def _report_no_chall_path(challbs: List[messages.ChallengeBody]) -> errors.AuthorizationError:
     """Logs and return a raisable error reporting that no satisfiable chall path exists.
 
