@@ -61,7 +61,7 @@ CLI_DEFAULTS: Dict[str, Any] = dict(  # noqa
     break_my_certs=False,
     rsa_key_size=2048,
     elliptic_curve="secp256r1",
-    key_type="rsa",
+    key_type="ecdsa",
     must_staple=False,
     redirect=None,
     auto_hsts=False,
@@ -80,6 +80,7 @@ CLI_DEFAULTS: Dict[str, Any] = dict(  # noqa
     eab_hmac_key=None,
     eab_kid=None,
     issuance_timeout=90,
+    run_deploy_hooks=False,
 
     # Subparsers
     num=None,
@@ -112,7 +113,6 @@ CLI_DEFAULTS: Dict[str, Any] = dict(  # noqa
     manual=False,
     webroot=False,
     dns_cloudflare=False,
-    dns_cloudxns=False,
     dns_digitalocean=False,
     dns_dnsimple=False,
     dns_dnsmadeeasy=False,
