@@ -2,40 +2,31 @@
 import itertools
 import logging
 import sys
-from collections.abc import Mapping
+from typing import Callable
+from typing import cast
+from typing import Dict
+from typing import Iterable
+from typing import Iterator
+from typing import List
+from typing import Mapping
+from typing import Optional
+from typing import Type
+from typing import Union
 
 import pkg_resources
-import zope.interface
-import zope.interface.verify
 
-from acme.magic_typing import Dict
+from certbot import configuration
 from certbot import errors
 from certbot import interfaces
 from certbot._internal import constants
 from certbot.compat import os
+from certbot.errors import Error
 
 logger = logging.getLogger(__name__)
 
-PREFIX_FREE_DISTRIBUTIONS = [
-    "certbot",
-    "certbot-apache",
-    "certbot-dns-cloudflare",
-    "certbot-dns-cloudxns",
-    "certbot-dns-digitalocean",
-    "certbot-dns-dnsimple",
-    "certbot-dns-dnsmadeeasy",
-    "certbot-dns-gehirn",
-    "certbot-dns-google",
-    "certbot-dns-linode",
-    "certbot-dns-luadns",
-    "certbot-dns-nsone",
-    "certbot-dns-ovh",
-    "certbot-dns-rfc2136",
-    "certbot-dns-route53",
-    "certbot-dns-sakuracloud",
-    "certbot-nginx",
-]
-"""Distributions for which prefix will be omitted."""
+
+PLUGIN_INTERFACES = [interfaces.Authenticator, interfaces.Installer, interfaces.Plugin]
+"""Interfaces that should be listed in `certbot plugins` output"""
 
 
 class PluginEntryPoint:
@@ -44,107 +35,77 @@ class PluginEntryPoint:
     # this object is mutable, don't allow it to be hashed!
     __hash__ = None  # type: ignore
 
-    def __init__(self, entry_point, with_prefix=False):
-        self.name = self.entry_point_to_plugin_name(entry_point, with_prefix)
-        self.plugin_cls = entry_point.load()
+    def __init__(self, entry_point: pkg_resources.EntryPoint) -> None:
+        self.name = self.entry_point_to_plugin_name(entry_point)
+        self.plugin_cls: Type[interfaces.Plugin] = entry_point.load()
         self.entry_point = entry_point
-        self.warning_message = None
-        self._initialized = None
-        self._prepared = None
-        self._hidden = False
-        self._long_description = None
+        self.warning_message: Optional[str] = None
+        self._initialized: Optional[interfaces.Plugin] = None
+        self._prepared: Optional[Union[bool, Error]] = None
 
-    def check_name(self, name):
+    def check_name(self, name: Optional[str]) -> bool:
         """Check if the name refers to this plugin."""
         if name == self.name:
-            if self.warning_message:
-                logger.warning(self.warning_message)
             return True
         return False
 
     @classmethod
-    def entry_point_to_plugin_name(cls, entry_point, with_prefix):
+    def entry_point_to_plugin_name(cls, entry_point: pkg_resources.EntryPoint) -> str:
         """Unique plugin name for an ``entry_point``"""
-        if with_prefix:
-            return entry_point.dist.key + ":" + entry_point.name
         return entry_point.name
 
     @property
-    def description(self):
+    def description(self) -> str:
         """Description of the plugin."""
         return self.plugin_cls.description
 
     @property
-    def description_with_name(self):
+    def description_with_name(self) -> str:
         """Description with name. Handy for UI."""
         return "{0} ({1})".format(self.description, self.name)
 
     @property
-    def long_description(self):
+    def long_description(self) -> str:
         """Long description of the plugin."""
-        if self._long_description:
-            return self._long_description
-        try:
-            return self.plugin_cls.long_description
-        except AttributeError:
-            return self.description
-
-    @long_description.setter
-    def long_description(self, description):
-        self._long_description = description
+        return getattr(self.plugin_cls, "long_description", self.description)
 
     @property
-    def hidden(self):
+    def hidden(self) -> bool:
         """Should this plugin be hidden from UI?"""
-        return self._hidden or getattr(self.plugin_cls, "hidden", False)
+        return getattr(self.plugin_cls, "hidden", False)
 
-    @hidden.setter
-    def hidden(self, hide):
-        self._hidden = hide
-
-    def ifaces(self, *ifaces_groups):
+    def ifaces(self, *ifaces_groups: Iterable[Type]) -> bool:
         """Does plugin implements specified interface groups?"""
         return not ifaces_groups or any(
-            all(iface.implementedBy(self.plugin_cls)
+            all(issubclass(self.plugin_cls, iface)
                 for iface in ifaces)
             for ifaces in ifaces_groups)
 
     @property
-    def initialized(self):
+    def initialized(self) -> bool:
         """Has the plugin been initialized already?"""
         return self._initialized is not None
 
-    def init(self, config=None):
+    def init(self, config: Optional[configuration.NamespaceConfig] = None) -> interfaces.Plugin:
         """Memoized plugin initialization."""
-        if not self.initialized:
+        if not self._initialized:
             self.entry_point.require()  # fetch extras!
+            # For plugins implementing ABCs Plugin, Authenticator or Installer, the following
+            # line will raise an exception if some implementations of abstract methods are missing.
             self._initialized = self.plugin_cls(config, self.name)
         return self._initialized
 
-    def verify(self, ifaces):
-        """Verify that the plugin conforms to the specified interfaces."""
-        assert self.initialized
-        for iface in ifaces:  # zope.interface.providedBy(plugin)
-            try:
-                zope.interface.verify.verifyObject(iface, self.init())
-            except zope.interface.exceptions.BrokenImplementation as error:
-                if iface.implementedBy(self.plugin_cls):
-                    logger.debug(
-                        "%s implements %s but object does not verify: %s",
-                        self.plugin_cls, iface.__name__, error, exc_info=True)
-                return False
-        return True
-
     @property
-    def prepared(self):
+    def prepared(self) -> bool:
         """Has the plugin been prepared already?"""
         if not self.initialized:
             logger.debug(".prepared called on uninitialized %r", self)
         return self._prepared is not None
 
-    def prepare(self):
+    def prepare(self) -> Union[bool, Error]:
         """Memoized plugin preparation."""
-        assert self.initialized
+        if self._initialized is None:
+            raise ValueError("Plugin is not initialized.")
         if self._prepared is None:
             try:
                 self._initialized.prepare()
@@ -160,35 +121,37 @@ class PluginEntryPoint:
                 self._prepared = error
             else:
                 self._prepared = True
-        return self._prepared
+        # Mypy seems to fail to understand the actual type here, let's help it.
+        return cast(Union[bool, Error], self._prepared)
 
     @property
-    def misconfigured(self):
+    def misconfigured(self) -> bool:
         """Is plugin misconfigured?"""
         return isinstance(self._prepared, errors.MisconfigurationError)
 
     @property
-    def problem(self):
+    def problem(self) -> Optional[Exception]:
         """Return the Exception raised during plugin setup, or None if all is well"""
         if isinstance(self._prepared, Exception):
             return self._prepared
         return None
 
     @property
-    def available(self):
+    def available(self) -> bool:
         """Is plugin available, i.e. prepared or misconfigured?"""
         return self._prepared is True or self.misconfigured
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "PluginEntryPoint#{0}".format(self.name)
 
-    def __str__(self):
+    def __str__(self) -> str:
         lines = [
             "* {0}".format(self.name),
             "Description: {0}".format(self.plugin_cls.description),
             "Interfaces: {0}".format(", ".join(
-                iface.__name__ for iface in zope.interface.implementedBy(
-                    self.plugin_cls))),
+                iface.__name__ for iface in PLUGIN_INTERFACES
+                if issubclass(self.plugin_cls, iface)
+            )),
             "Entry point: {0}".format(self.entry_point),
         ]
 
@@ -203,7 +166,7 @@ class PluginEntryPoint:
 class PluginsRegistry(Mapping):
     """Plugins registry."""
 
-    def __init__(self, plugins):
+    def __init__(self, plugins: Mapping[str, PluginEntryPoint]) -> None:
         # plugins are sorted so the same order is used between runs.
         # This prevents deadlock caused by plugins acquiring a lock
         # and ensures at least one concurrent Certbot instance will run
@@ -211,9 +174,9 @@ class PluginsRegistry(Mapping):
         self._plugins = dict(sorted(plugins.items()))
 
     @classmethod
-    def find_all(cls):
+    def find_all(cls) -> 'PluginsRegistry':
         """Find plugins using setuptools entry points."""
-        plugins = {}  # type: Dict[str, PluginEntryPoint]
+        plugins: Dict[str, PluginEntryPoint] = {}
         plugin_paths_string = os.getenv('CERTBOT_PLUGIN_PATH')
         plugin_paths = plugin_paths_string.split(':') if plugin_paths_string else []
         # XXX should ensure this only happens once
@@ -226,85 +189,68 @@ class PluginsRegistry(Mapping):
             pkg_resources.iter_entry_points(
                 constants.OLD_SETUPTOOLS_PLUGINS_ENTRY_POINT),)
         for entry_point in entry_points:
-            plugin_ep = cls._load_entry_point(entry_point, plugins, with_prefix=False)
-            # entry_point.dist cannot be None here, we would have blown up
-            # earlier, however, this assertion is needed for mypy.
-            assert entry_point.dist is not None
-            if entry_point.dist.key not in PREFIX_FREE_DISTRIBUTIONS:
-                prefixed_plugin_ep = cls._load_entry_point(entry_point, plugins, with_prefix=True)
-                prefixed_plugin_ep.hidden = True
-                message = (
-                    "Plugin legacy name {0} may be removed in a future version. "
-                    "Please use {1} instead.").format(prefixed_plugin_ep.name, plugin_ep.name)
-                prefixed_plugin_ep.warning_message = message
-                prefixed_plugin_ep.long_description = "(WARNING: {0}) {1}".format(
-                    message, prefixed_plugin_ep.long_description)
+            cls._load_entry_point(entry_point, plugins)
 
         return cls(plugins)
 
     @classmethod
-    def _load_entry_point(cls, entry_point, plugins, with_prefix):
-        plugin_ep = PluginEntryPoint(entry_point, with_prefix)
+    def _load_entry_point(cls, entry_point: pkg_resources.EntryPoint,
+                          plugins: Dict[str, PluginEntryPoint]) -> None:
+        plugin_ep = PluginEntryPoint(entry_point)
         if plugin_ep.name in plugins:
             other_ep = plugins[plugin_ep.name]
+            plugin1 = plugin_ep.entry_point.dist.key if plugin_ep.entry_point.dist else "unknown"
+            plugin2 = other_ep.entry_point.dist.key if other_ep.entry_point.dist else "unknown"
             raise Exception("Duplicate plugin name {0} from {1} and {2}.".format(
-                plugin_ep.name, plugin_ep.entry_point.dist.key, other_ep.entry_point.dist.key))
-        if interfaces.IPluginFactory.providedBy(plugin_ep.plugin_cls):
+                plugin_ep.name, plugin1, plugin2))
+        if issubclass(plugin_ep.plugin_cls, interfaces.Plugin):
             plugins[plugin_ep.name] = plugin_ep
         else:  # pragma: no cover
             logger.warning(
-                "%r does not provide IPluginFactory, skipping", plugin_ep)
+                "%r does not inherit from Plugin, skipping", plugin_ep)
 
-        return plugin_ep
-
-    def __getitem__(self, name):
+    def __getitem__(self, name: str) -> PluginEntryPoint:
         return self._plugins[name]
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[str]:
         return iter(self._plugins)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(self._plugins)
 
-    def init(self, config):
+    def init(self, config: configuration.NamespaceConfig) -> List[interfaces.Plugin]:
         """Initialize all plugins in the registry."""
         return [plugin_ep.init(config) for plugin_ep
                 in self._plugins.values()]
 
-    def filter(self, pred):
+    def filter(self, pred: Callable[[PluginEntryPoint], bool]) -> "PluginsRegistry":
         """Filter plugins based on predicate."""
         return type(self)({name: plugin_ep for name, plugin_ep
-                               in self._plugins.items() if pred(plugin_ep)})
+                           in self._plugins.items() if pred(plugin_ep)})
 
-    def visible(self):
+    def visible(self) -> "PluginsRegistry":
         """Filter plugins based on visibility."""
         return self.filter(lambda plugin_ep: not plugin_ep.hidden)
 
-    def ifaces(self, *ifaces_groups):
+    def ifaces(self, *ifaces_groups: Iterable[Type]) -> "PluginsRegistry":
         """Filter plugins based on interfaces."""
         return self.filter(lambda p_ep: p_ep.ifaces(*ifaces_groups))
 
-    def verify(self, ifaces):
-        """Filter plugins based on verification."""
-        return self.filter(lambda p_ep: p_ep.verify(ifaces))
-
-    def prepare(self):
+    def prepare(self) -> List[Union[bool, Error]]:
         """Prepare all plugins in the registry."""
         return [plugin_ep.prepare() for plugin_ep in self._plugins.values()]
 
-    def available(self):
+    def available(self) -> "PluginsRegistry":
         """Filter plugins based on availability."""
         return self.filter(lambda p_ep: p_ep.available)
         # successfully prepared + misconfigured
 
-    def find_init(self, plugin):
+    def find_init(self, plugin: interfaces.Plugin) -> Optional[PluginEntryPoint]:
         """Find an initialized plugin.
 
-        This is particularly useful for finding a name for the plugin
-        (although `.IPluginFactory.__call__` takes ``name`` as one of
-        the arguments, ``IPlugin.name`` is not part of the interface)::
+        This is particularly useful for finding a name for the plugin::
 
-          # plugin is an instance providing IPlugin, initialized
+          # plugin is an instance providing Plugin, initialized
           # somewhere else in the code
           plugin_registry.find_init(plugin).name
 
@@ -319,12 +265,12 @@ class PluginsRegistry(Mapping):
             return candidates[0]
         return None
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "{0}({1})".format(
             self.__class__.__name__, ','.join(
                 repr(p_ep) for p_ep in self._plugins.values()))
 
-    def __str__(self):
+    def __str__(self) -> str:
         if not self._plugins:
             return "No plugins"
         return "\n\n".join(str(p_ep) for p_ep in self._plugins.values())
