@@ -1,7 +1,6 @@
 """Utilities for all Certbot."""
 import argparse
 import atexit
-import collections
 import errno
 import logging
 import platform
@@ -14,10 +13,10 @@ from typing import Callable
 from typing import Dict
 from typing import IO
 from typing import List
+from typing import NamedTuple
 from typing import Optional
 from typing import Set
 from typing import Tuple
-from typing import TYPE_CHECKING
 from typing import Union
 import warnings
 
@@ -33,15 +32,21 @@ _USE_DISTRO = sys.platform.startswith('linux')
 if _USE_DISTRO:
     import distro
 
-if TYPE_CHECKING:
-    import distutils.version # pylint: disable=deprecated-module
-
 logger = logging.getLogger(__name__)
 
 
-Key = collections.namedtuple("Key", "file pem")
-# Note: form is the type of data, "pem" or "der"
-CSR = collections.namedtuple("CSR", "file data form")
+class Key(NamedTuple):
+    """Container for an optional file path and contents for a PEM-formated private key."""
+    file: Optional[str]
+    pem: bytes
+
+
+class CSR(NamedTuple):
+    """Container for an optional file path and contents for a PEM or DER-formatted CSR."""
+    file: Optional[str]
+    data: bytes
+    # Note: form is the type of data, "pem" or "der"
+    form: str
 
 
 # ANSI SGR escape codes
@@ -611,24 +616,6 @@ def is_wildcard_domain(domain: Union[str, bytes]) -> bool:
     return domain.startswith(b"*.")
 
 
-def get_strict_version(normalized: str) -> "distutils.version.StrictVersion":
-    """Converts a normalized version to a strict version.
-
-    :param str normalized: normalized version string
-
-    :returns: An equivalent strict version
-    :rtype: distutils.version.StrictVersion
-
-    """
-    warnings.warn("certbot.util.get_strict_version is deprecated and will be "
-                  "removed in a future release.", DeprecationWarning)
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", DeprecationWarning)
-        import distutils.version # pylint: disable=deprecated-module
-        # strict version ending with "a" and a number designates a pre-release
-        return distutils.version.StrictVersion(normalized.replace(".dev", "a"))
-
-
 def is_staging(srv: str) -> bool:
     """
     Determine whether a given ACME server is a known test / staging server.
